//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "Event.hpp"
#include "MyException.hpp"
#include "Screen.hpp"
#include "Global.hpp"
#include "nbStrConvert.hpp"
#include "PseudoEvent.hpp"

#include <SFML/Graphics.hpp>

#include <lua.hpp>
#include <luabind/luabind.hpp>

#include <iostream>
#include <algorithm>
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
namespace KeyIds
{
    const int JoystickIdStart = 4000, JoystickSpan = 1000; // Span : Id difference between 2 Joystick => Max number of button
    const std::string JoystickName = "Joystick";
    const int UnknownId = 2000;
    const char* UnknownStr = "Unknown";

    // Mouse buttons have negative ids
    const int Key_All = -2000, MouseWheelUp = -3000, MouseWheelDown = -3001;
    const int keyIdTbl[] = {Key_All, UnknownId,  MouseWheelUp, MouseWheelDown, - sf::Mouse::Left - 1000, -sf::Mouse::Right - 1000, -sf::Mouse::Middle - 1000, -sf::Mouse::XButton1 - 1000, -sf::Mouse::XButton2 - 1000, sf::Key::A, sf::Key::B, sf::Key::C, sf::Key::D, sf::Key::E, sf::Key::F, sf::Key::G, sf::Key::H, sf::Key::I, sf::Key::J, sf::Key::K, sf::Key::L, sf::Key::M,sf::Key::N, sf::Key::O, sf::Key::P, sf::Key::Q,sf::Key::R, sf::Key::S, sf::Key::T, sf::Key::U, sf::Key::V, sf::Key::W, sf::Key::X, sf::Key::Y, sf::Key::Z, sf::Key::Num0, sf::Key::Num1, sf::Key::Num2, sf::Key::Num3, sf::Key::Num4, sf::Key::Num5, sf::Key::Num6, sf::Key::Num7, sf::Key::Num8, sf::Key::Num9, sf::Key::Escape, sf::Key::LControl, sf::Key::LShift, sf::Key::LAlt, sf::Key::LSystem, sf::Key::RControl, sf::Key::RShift, sf::Key::RAlt, sf::Key::RSystem, sf::Key::Menu, sf::Key::LBracket, sf::Key::RBracket, sf::Key::SemiColon, sf::Key::Comma, sf::Key::Period, sf::Key::Quote, sf::Key::Slash, sf::Key::BackSlash, sf::Key::Tilde, sf::Key::Equal, sf::Key::Dash, sf::Key::Space, sf::Key::Return, sf::Key::Back, sf::Key::Tab, sf::Key::PageUp, sf::Key::PageDown, sf::Key::End, sf::Key::Home, sf::Key::Insert, sf::Key::Delete, sf::Key::Add, sf::Key::Subtract, sf::Key::Multiply, sf::Key::Divide, sf::Key::Left, sf::Key::Right, sf::Key::Up, sf::Key::Down, sf::Key::Numpad0, sf::Key::Numpad1, sf::Key::Numpad2, sf::Key::Numpad3, sf::Key::Numpad4, sf::Key::Numpad5, sf::Key::Numpad6, sf::Key::Numpad7, sf::Key::Numpad8, sf::Key::Numpad9, sf::Key::F1, sf::Key::F2, sf::Key::F3, sf::Key::F4, sf::Key::F5, sf::Key::F6, sf::Key::F7, sf::Key::F8, sf::Key::F9, sf::Key::F10, sf::Key::F11, sf::Key::F12, sf::Key::F13, sf::Key::F14, sf::Key::F15, sf::Key::Pause};
    const char* keyNameTbl[] = {"", UnknownStr, "MouseWheelUp", "MouseWheelDown", "MouseLeft", "MouseRight", "MouseMiddle", "MouseXButton1", "MouseXButton2", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "Num0", "Num1", "Num2", "Num3", "Num4", "Num5", "Num6", "Num7", "Num8", "Num9", "Escape", "LControl", "LShift", "LAlt", "LSystem", "RControl", "RShift", "RAlt", "RSystem", "Menu", "LBracket", "RBracket", "SemiColon", "Comma", "Period", "Quote", "Slash", "BackSlash", "Tilde", "Equal", "Dash", "Space", "Return", "Back", "Tab", "PageUp", "PageDown", "End", "Home", "Insert", "Delete", "Add", "Subtract", "Multiply", "Divide", "Left", "Right", "Up", "Down", "Numpad0", "Numpad1", "Numpad2", "Numpad3", "Numpad4", "Numpad5", "Numpad6", "Numpad7", "Numpad8", "Numpad9", "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10", "F11", "F12", "F13", "F14", "F15", "Pause"};
    const size_t keyCount = sizeof(keyIdTbl) / sizeof(int);
}
//------------------------------------------------------------------------------
using namespace std;
//------------------------------------------------------------------------------
bool IsKeyPressed(const std::string &keyName, const sf::Input &input)
{
    if(keyName.substr(0, KeyIds::JoystickName.size()) == KeyIds::JoystickName)
    {
        size_t pos = keyName.find('_');
        if(pos == string::npos)
        {
            int joyId = stringToNb<int>(keyName.substr(KeyIds::JoystickName.size(), pos - KeyIds::JoystickName.size()));
            int joyBtId = stringToNb<int>(keyName.substr(pos + 1, pos - keyName.size() - pos - 1));

            return input.IsJoystickButtonDown(joyId, joyBtId);
        }
        else
        {
            MY_THROW("Unknown Joystick '" << keyName << "' !");
        }
    }
    else
    {
        int keyId = ButtonEventHandler::KeyNameToId(keyName);

        if(keyId == KeyIds::UnknownId)
        {
            MY_THROW("Unknown key '" << keyName << "' !");
        }
        else if(keyId > 0) // Key
        {
            return input.IsKeyDown(sf::Key::Code(keyId));
        }
        else // Mouse buttons have negative ids
        {
            return input.IsMouseButtonDown(sf::Mouse::Button(- keyId - 1000));
        }
    }
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
EventHandlerBase::EventHandlerBase(const luabind::object &luaFunction) : m_isEndOfLife(false)
{
    if(luaFunction.is_valid())
    {
        if(luabind::type(luaFunction) == LUA_TFUNCTION)
        {
            m_luaFunction = luaFunction;
        }
        else if(luabind::type(luaFunction) != LUA_TNIL)
        {
            MY_THROW("Function: Bad type '" << luabind::type(luaFunction) << "' !");
        }
    }
}
//------------------------------------------------------------------------------
void EventHandlerBase::Destroy()
{
    if(m_isEndOfLife)
    {
        cerr << "EventHandlerBase::Destroy(): Already end of life !!" << endl;
    }
    m_isEndOfLife = true;
}
//------------------------------------------------------------------------------
/** Set user data */
void EventHandlerBase::SetUserData(const std::string &id, const luabind::object &data)
{
    m_userData[id] = data;
}
//------------------------------------------------------------------------------
/** Get user data */
luabind::object EventHandlerBase::GetUserData(const std::string &id)
{
    std::map<std::string, luabind::object>::iterator it = m_userData.find(id);

    if(it != m_userData.end())
    {
        return it->second;
    }
    else
    {
        MY_THROW("Object::GetUserData : Key '" << id << "' not found in event !");
    }
}

//------------------------------------------------------------------------------
/** Return true if the user data exists, false otherwise */
bool EventHandlerBase::UserDataExists(const std::string &id)
{
    return(m_userData.find(id) != m_userData.end());
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
EventHandler::EventHandler(const luabind::object &luaFunction) : EventHandlerBase(luaFunction)
{
    //ctor
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
std::map<int, std::string> ButtonEventHandler::m_mapIdToName = std::map<int, std::string>();
std::map<std::string, int> ButtonEventHandler::m_mapNameToId = std::map<std::string, int>();

//------------------------------------------------------------------------------
ButtonEventHandler::ButtonEventHandler(const luabind::object &luaFunction, const std::string &keyName, bool pressed) : EventHandler(luaFunction), m_pressed(pressed), m_keyName(keyName), m_evtFn(&SendEvent)
{
    if(keyName.substr(0, KeyIds::JoystickName.size()) == KeyIds::JoystickName)
    {
        size_t pos = keyName.find('_');
        if(pos == string::npos)
        {
            int joyId = stringToNb<int>(keyName.substr(KeyIds::JoystickName.size(), pos - KeyIds::JoystickName.size()));
            int joyBtId = stringToNb<int>(keyName.substr(pos + 1, pos - keyName.size() - pos - 1));

            m_keyId = KeyIds::JoystickIdStart + joyId * KeyIds::JoystickSpan + joyBtId;
            m_keyName = KeyIds::JoystickName + nbToString(joyId) + string("_") + nbToString(joyBtId);
        }
    }
    else
    {
        m_keyId = KeyNameToId(keyName);
    }
}
//------------------------------------------------------------------------------
void ButtonEventHandler::SendEvent(const std::string &keyName, bool pressed, ButtonEventHandler *obj, const luabind::object &luaFunction)
{
    if(! luaFunction.is_valid())
    {
        return;
    }

    try
    {
        luabind::call_function<void>(luaFunction, keyName, pressed, static_cast<EventHandlerBase*>(obj));
    }
    MY_CATCH_LUA_CERR()

}
//------------------------------------------------------------------------------
void ButtonEventHandler::SetEventFunction(void(*evtFn)(const std::string &, bool, ButtonEventHandler*, const luabind::object &))
{
    if(evtFn)
    {
        m_evtFn = evtFn;
    }
    else
    {
        evtFn = &SendEvent;
    }
}
//------------------------------------------------------------------------------
bool ButtonEventHandler::ParseEvent(const sf::Event &event, std::string &keyName, bool &pressed)
{
    if((m_pressed && event.Type == sf::Event::KeyPressed) || (!m_pressed && event.Type == sf::Event::KeyReleased))
    {
        if(m_keyId == KeyIds::Key_All)
        {
            keyName = KeyIdToName(event.Key.Code);
            pressed = m_pressed;
            return true;
        }
        else if(event.Key.Code == m_keyId)
        {
            keyName = m_keyName;
            pressed = m_pressed;
            return true;
        }
        else
        {
            return false;
        }
    }
    else if((m_pressed && event.Type == sf::Event::MouseButtonPressed) || (!m_pressed && event.Type == sf::Event::MouseButtonReleased))
    {
        const int code = -(event.MouseButton.Button + 1000);
        if(m_keyId == KeyIds::Key_All)
        {
            keyName = KeyIdToName(code);
            pressed = m_pressed;
            return true;
        }
        else if(code == m_keyId)
        {
            keyName = m_keyName;
            pressed = m_pressed;
            return true;
        }
        else
        {
            return false;
        }
    }
    else if(m_pressed && event.Type == sf::Event::MouseWheelMoved)
    {
        if(m_keyId == KeyIds::Key_All)
        {
            std::string keyName = event.MouseWheel.Delta >= 0 ? KeyIdToName(KeyIds::MouseWheelUp) : KeyIdToName(KeyIds::MouseWheelDown);
            for(int i = 0; i < abs(event.MouseWheel.Delta); i++)
            {
                keyName = keyName;
                pressed = m_pressed;
            }
            return true;
        }
        else if((m_keyId == KeyIds::MouseWheelUp && event.MouseWheel.Delta > 0) || (m_keyId == KeyIds::MouseWheelDown && event.MouseWheel.Delta < 0))
        {
            for(int i = 0; i < abs(event.MouseWheel.Delta); i++)
            {
                keyName = m_keyName;
                pressed = m_pressed;
            }
            return true;
        }
        else
        {
            return false;
        }
    }
    else if((m_pressed && event.Type == sf::Event::JoyButtonPressed) || (!m_pressed && event.Type == sf::Event::JoyButtonReleased))
    {
        if(m_keyId == KeyIds::Key_All)
        {
            int joyId = (m_keyId - KeyIds::JoystickIdStart) % KeyIds::JoystickSpan;
            int joyBtId = m_keyId - (KeyIds::JoystickIdStart + joyId * KeyIds::JoystickSpan);

            keyName = KeyIds::JoystickName + nbToString(joyId) + string("_") + nbToString(joyBtId);
            pressed = m_pressed;
            return true;
        }
        else
        {
            int evtId = KeyIds::JoystickIdStart + event.JoyButton.JoystickId * KeyIds::JoystickSpan + event.JoyButton.Button;

            if(evtId == m_keyId)
            {
                keyName = m_keyName;
                pressed = m_pressed;
                return true;
            }
            else
            {
                return false;
            }
        }
    }
    else
    {
        return false;
    }
}
//------------------------------------------------------------------------------
bool ButtonEventHandler::Match(const sf::Event &event, const Screen *screen, const sf::RenderWindow &App)
{
    string keyName;
    bool pressed;

    if(ParseEvent(event, keyName, pressed))
    {
        (*m_evtFn)(keyName, pressed, this, m_luaFunction);
        return true;
    }
    else
    {
        return false;
    }
}
//------------------------------------------------------------------------------
void ButtonEventHandler::FillMaps()
{
    m_mapIdToName.clear();
    m_mapNameToId.clear();
    for(size_t i = 0; i < KeyIds::keyCount; i++)
    {
        m_mapIdToName[KeyIds::keyIdTbl[i]] = KeyIds::keyNameTbl[i];
        m_mapNameToId[KeyIds::keyNameTbl[i]] = KeyIds::keyIdTbl[i];
    }
}
//------------------------------------------------------------------------------
int ButtonEventHandler::KeyNameToId(const std::string &name)
{
    std::map<int, std::string> m_mapIdToName;
    std::map<std::string, int>::iterator it = m_mapNameToId.find(name);
    if(it != m_mapNameToId.end())
    {
        return it->second;
    }
    else
    {
         cerr << "KeyNameToId : Name '" << name << "'not found !" << endl;
         return KeyIds::UnknownId;
    }
}
//------------------------------------------------------------------------------
std::string ButtonEventHandler::KeyIdToName(int id)
{
    std::map<int, std::string>::iterator it = m_mapIdToName.find(id);
    if(it != m_mapIdToName.end())
    {
        return it->second;
    }
    else
    {
         cerr << "KeyNameToId : Id '" << id << "'not found !" << endl;
         return KeyIds::UnknownStr;
    }
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
MouseMoveEventHandler::MouseMoveEventHandler(const luabind::object &luaFunction) : EventHandler(luaFunction)
{

}
//------------------------------------------------------------------------------
bool MouseMoveEventHandler::Match(const sf::Event &event, const Screen *screen, const sf::RenderWindow &App)
{
    if(event.Type == sf::Event::MouseMoved)
    {
        if(screen->IsMouseRelative())
        {
            SendEvent(Vector2D(float(event.MouseMove.X) - float(App.GetWidth() / 2), float(event.MouseMove.Y) - float(App.GetHeight() / 2)), this, m_luaFunction);
            return true;
        }
        else
        {
            SendEvent(App.ConvertCoords(event.MouseMove.X, event.MouseMove.Y), this, m_luaFunction);
            return true;
        }
    }
    else
    {
        return false;
    }
}
//------------------------------------------------------------------------------
void MouseMoveEventHandler::SendEvent(const Vector2D &pos, MouseMoveEventHandler *obj, const luabind::object &luaFunction)
{
    if(! luaFunction.is_valid())
    {
        return;
    }

    try
    {
        luabind::call_function<void>(luaFunction, pos, static_cast<EventHandlerBase*>(obj));
    }
    MY_CATCH_LUA_CERR()

}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
ButtonInZoneEventHandler::ButtonInZoneEventHandler(const luabind::object &luaFunction, MouseZoneEvent *zoneEvt, const std::string &keyName, bool pressed) : ButtonEventHandler(luaFunction, keyName, pressed), m_zoneEvt(zoneEvt)
{

}
//------------------------------------------------------------------------------
bool ButtonInZoneEventHandler::Match(const sf::Event &event, const Screen *screen, const sf::RenderWindow &App)
{
    string keyName;
    bool pressed;

    if(m_zoneEvt->IsInside() && ParseEvent(event, keyName, pressed))
    {
        SendEvent(GetKeyName(), GetPressed(), m_zoneEvt, this, m_luaFunction);
        return true;
    }
    else
    {
        return false;
    }
}
//------------------------------------------------------------------------------
void ButtonInZoneEventHandler::SendEvent(const std::string &keyName, bool pressed, MouseZoneEvent *zoneEvtObj, ButtonInZoneEventHandler *obj, const luabind::object &luaFunction)
{
    if(! luaFunction.is_valid())
    {
        return;
    }

    try
    {
        luabind::call_function<void>(luaFunction, keyName, pressed, zoneEvtObj, static_cast<EventHandlerBase*>(obj));
    }
    MY_CATCH_LUA_CERR()
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
} // namespace En
