//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "PhLine.hpp"
//------------------------------------------------------------------------------
#include <iostream>
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
using namespace std;
//------------------------------------------------------------------------------
PhLine::PhLine(Group *parent, int phGroupId, const std::string &name, const Vector2D &pt1, const Vector2D &pt2) : PhysicObjectLine(parent, phGroupId, name)
{
    LinePtPt line(pt1, pt2);

    m_size = line.GetSize();
    SetPosition(line.pt1);

    Vector2D vect = line.pt2 - line.pt1;
    SetRotation(- 2 * atan(vect.y / (vect.x + m_size)));

    RefreshMaxInterRadius();
}
//------------------------------------------------------------------------------
bool PhLine::IsPointInside(const Vector2D &point)
{
    return false;
}
//------------------------------------------------------------------------------
std::vector<Vector2D> PhLine::GetAllPoints() const
{
    Vector2D pos = GetGlobalPosition();

    std::vector<Vector2D> ptList;
    ptList.resize(2);
    ptList[0] = pos;
    ptList[1] = pos + PointRotate(Vector2D(0, 0), Vector2D(m_size, 0), GetGlobalRotation());

    return ptList;
}
//------------------------------------------------------------------------------
void PhLine::SetVector(const Vector2D &vect)
{
    m_size = LinePtVect(Vector2D(0, 0), vect).GetSize();
    SetRotation(- 2 * atan(vect.y / (vect.x + m_size)));

    RefreshMaxInterRadius();
}
//------------------------------------------------------------------------------
Vector2D PhLine::GetVector() const
{
    return PointRotate(Vector2D(0, 0), Vector2D(m_size, 0), GetGlobalRotation());
}
//------------------------------------------------------------------------------
void PhLine::SetPt2(const Vector2D &pt2)
{
    LinePtPt line(GetPosition(), pt2);
    m_size = line.GetSize();

    Vector2D vect = line.pt2 - line.pt1;
    SetRotation(- 2 * atan(vect.y / (vect.x + m_size)));

    RefreshMaxInterRadius();
}
//------------------------------------------------------------------------------
Vector2D PhLine::GetPt2() const
{
    return (GetPosition() + PointRotate(Vector2D(0, 0), Vector2D(m_size, 0), GetGlobalRotation()));
}
//------------------------------------------------------------------------------
/*void PhLine::RegisterClassLua(lua_State *myLuaState)
{
    luabind::module(myLuaState)
    [
        luabind::class_<Object>("Object")

    ];
}*/
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
