//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef PHLINE_HPP
#define PHLINE_HPP
//------------------------------------------------------------------------------
#include "PhysicObject.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
/// LUA_HEADER_COPY_ON ///
/** A physic object representing a line created using Group::CreatePhLine. */
/// LUA_HEADER_COPY_OFF ///
class PhLine : public PhysicObjectLine
{
public:
    PhLine(Group *parent, int phGroupId, const std::string &name, const Vector2D &pt1, const Vector2D &pt2);
    static PhLine *Create(Group *parent, int phGroupId, const std::string &name, const Vector2D &pt1, const Vector2D &pt2) {return new PhLine(parent, phGroupId, name, pt1, pt2);}

    /// LUA_EXPORT_ON(PhLine, PhysicObject) ///
    // Pt2
    /** Access the line vector */
    //LUA// luaProperty(sf::Vecor2f, vector, GetVector, SetVector);
    /** Set the line vector */
    void SetVector(const Vector2D &vector);
    /** Get the line vector */
    Vector2D GetVector() const;

    /** Access the line's second point (the first one is the position) */
    //LUA// luaProperty(sf::Vecor2f, pt2, GetPt2, SetPt2);
    /** Set the line's second point (the first one is the position) */
    void SetPt2(const Vector2D &pt2);
    /** Get the line's second point (the first one is the position) */
    Vector2D GetPt2() const;

    // Clone & Create
    /** Create a clone of the object and return an handler to the clone. */
    virtual PhLine *Clone() const {return CloneToGroup(NULL);}

    /** Create a clone of the object, change the parent and return an handler to the clone. */
    virtual PhLine *CloneToGroup(Group *newParent) const {PhLine *obj = new PhLine(*this); obj->RegisterClone(newParent); return obj;}
    /// LUA_EXPORT_OFF ///

    virtual bool IsPointInside(const Vector2D &point);

    // No Lua
    // Lua
    //static void RegisterClassLua(lua_State *myLuaState);

    // Type
    ADD_GET_TYPE_FN(OBJ_TYPE_PHYSIC_LINE)

    // All pts
    virtual std::vector<Vector2D> GetAllPoints() const;

protected:
    float m_size;
};
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
#endif // PHLINE_HPP
