//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "PhShape.hpp"

#include "MyException.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
PhShape::PhShape(Group *parent, int phGroupId, const std::string &name) : PhysicObjectLine(parent, phGroupId, name)
{
    //ctor
    RefreshMaxInterRadius();
}
//------------------------------------------------------------------------------
Vector2D PhShape::GetPoint(int nb) const
{
    if(nb < 0 || nb > int(m_pts.size()))
    {
        MY_THROW("PhShape::GetPoint : Number is not included in point list !");
    }

    return m_pts[nb];
}
//------------------------------------------------------------------------------
void PhShape::SetPoint(int nb, const Vector2D &pt)
{
    if(nb < 0 || nb > int(m_pts.size()))
    {
        MY_THROW("PhShape::GetPoint : Number is not included in point list !");
    }

    m_pts[nb] = pt;
    RefreshMaxInterRadius();
}
//------------------------------------------------------------------------------
void PhShape::RemovePoint(int nb)
{
    if(nb < 0 || nb > int(m_pts.size()))
    {
        MY_THROW("PhShape::RemovePoint : Number is not included in point list !");
    }

    m_pts.erase(m_pts.begin() + nb);
    RefreshMaxInterRadius();
}
//------------------------------------------------------------------------------
void PhShape::AddPoint(const Vector2D &pt)
{
    m_pts.push_back(pt);
    RefreshMaxInterRadius();
}
//------------------------------------------------------------------------------
std::vector<Vector2D> PhShape::GetAllPoints() const
{
    float angle = GetGlobalRotation();
    Vector2D pos = GetGlobalPosition();

    std::vector<Vector2D> ptList;
    ptList.resize(m_pts.size());

    for(size_t i = 0; i < m_pts.size(); i++)
    {
        ptList[i] = pos + PointRotate(Vector2D(0, 0), m_pts[i], angle);
    }

    return ptList;
}
//------------------------------------------------------------------------------
/*void PhShape::RegisterClassLua(lua_State *myLuaState)
{
    luabind::module(myLuaState)
    [
        luabind::class_<Object>("Object")

    ];
}*/
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
