//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef PHSHAPE_HPP
#define PHSHAPE_HPP
//------------------------------------------------------------------------------
#include "PhysicObject.hpp"
//------------------------------------------------------------------------------
namespace En
{
/// LUA_HEADER_COPY_ON ///
/** A physic object representing an user shape created using Group::CreatePhShape. */
/// LUA_HEADER_COPY_OFF ///
//------------------------------------------------------------------------------
class PhShape : public PhysicObjectLine
{
public:
    PhShape(Group *parent, int phGroupId, const std::string &name);
    static PhShape *Create(Group *parent, int phGroupId, const std::string &name) {return new PhShape(parent, phGroupId, name);}

    /// LUA_EXPORT_ON(PhShape, PhysicObject) ///
    // Access point
    /** Get the point \\a nb of the shape */
    Vector2D GetPoint(int nb) const;
    /** Set the point \\a nb of the shape */
    void SetPoint(int nb, const Vector2D &pt);
    /** Add a point to the shape */
    void AddPoint(const Vector2D &pt);
    /** Remove the point \\a nb from the shape */
    void RemovePoint(int nb);

    // Clone & Create
    /** Create a clone of the object and return an handler to the clone. */
    virtual PhShape *Clone() const {return CloneToGroup(NULL);}

    /** Create a clone of the object, change the parent and return an handler to the clone. */
    virtual PhShape *CloneToGroup(Group *newParent) const {PhShape *obj = new PhShape(*this); obj->RegisterClone(newParent); return obj;}
    /// LUA_EXPORT_OFF ///

    // No Lua
    // Lua
    //static void RegisterClassLua(lua_State *myLuaState);

    // Type
    ADD_GET_TYPE_FN(OBJ_TYPE_PHYSIC_SHAPE)

    virtual std::vector<Vector2D> GetAllPoints() const;
protected:
    std::vector<Vector2D> m_pts;

private:
};
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
#endif // PHSHAPE_HPP
