require("guiLib/common")

------------------------------------------------------------
-- Change the id of the string selected in the choice control
------------------------------------------------------------
function choiceControlChangeId(group, strId)
	local tbl = group:GetUserData("choiceList")
	
	if #tbl ~= 0 then
		if strId < 1 then
			strId = #tbl
		elseif strId > #tbl then
			strId = 1
		end
	
		-- previous bt
		local previousSpr = group:FindByNameSprite("btPrevious")
		local nextSpr = group:FindByNameSprite("btNext")
		local textObj = group:FindByNameText("text")

		textObj:SetText(tbl[strId])
		local txtSize = textObj:GetTextSize()
		textObj:SetPosition(Vector2D(previousSpr:GetSize().x + (nextSpr:GetPosition().x - previousSpr:GetSize().x) / 2 - txtSize.x / 2, previousSpr:GetSize().y / 2 - txtSize.y / 2))

		-- set value	
		group:SetUserData("currentId", strId)
	end
end

------------------------------------------------------------
-- Get the string table associated with the choice control
------------------------------------------------------------
function choiceControlGetTable(group)
	return group:GetUserData("choiceList")
end

------------------------------------------------------------
-- Get the number of string in the control
------------------------------------------------------------
function choiceControlGetCount(group)
	return #group:GetUserData("choiceList")
end

------------------------------------------------------------
-- Create a choice control
------------------------------------------------------------
function choiceControlCreate(group, width, fontSize)
	if width == nil then
		width = 200
	end

	-- list id
	group:SetUserData("choiceList", {})
	group:SetUserData("currentId", 0)
	
	-- previous bt
	tmp = group:CreateSprite("btPrevious")
	tmp:SetImage(arrowPreviousTex)
	tmp:SetPosition(Vector2D(0, 0))

	local btPreviousZone = group:CreatePhRect(0, "btPreviousZone", tmp:GetSize())
	btPreviousZone:SetPosition(tmp:GetPosition())

	-- next bt
	tmp = group:CreateSprite("btNext")
	tmp:SetImage(arrowNextTex)
	tmp:SetPosition(Vector2D(width - tmp:GetSize().x, 0))

	local btNextZone = group:CreatePhRect(0, "btNextZone", tmp:GetSize())
	btNextZone:SetPosition(tmp:GetPosition())

	-- Text
	tmp = group:CreateText("text", "")
	tmp:SetZpos(100) -- More than the buttons
	
	if fontSize ~= nil then
		tmp:SetSize(fontSize)
	end
	
	-- events
	local screenOfGroup = group:GetParentScreen()
	
	local zoneEvent =  screenOfGroup:RegisterMouseZoneEvent(nil, btPreviousZone)
	group:RegisterEventAutoRemoveOnDestroy(zoneEvent)
	local mouseClickEvent = screenOfGroup:RegisterButtonInZoneEvent(choiceControlApi_onClickOnPreviousNext, zoneEvent, "MouseLeft", true)
	group:RegisterEventAutoRemoveOnDestroy(mouseClickEvent)
	
	local zoneEvent = screenOfGroup:RegisterMouseZoneEvent(nil, btNextZone)
	group:RegisterEventAutoRemoveOnDestroy(zoneEvent)
	local mouseClickEvent = screenOfGroup:RegisterButtonInZoneEvent(choiceControlApi_onClickOnPreviousNext, zoneEvent, "MouseLeft", true)
	group:RegisterEventAutoRemoveOnDestroy(mouseClickEvent)
end

------------------------------------------------------------
-- Add a string to a choice control
------------------------------------------------------------
function choiceControlAddString(group, str)
	local tbl = group:GetUserData("choiceList")
	
	table.insert(tbl, str)
	
	if #tbl == 1 then -- The list was empty before the call => We change the current selection
		choiceControlChangeId(group, 1)
	end
end

------------------------------------------------------------
-- Add a strings to a choice control
------------------------------------------------------------
function choiceControlAddStringTbl(group, strTbl)
	for i = 1, #strTbl do
		choiceControlAddString(group, strTbl[i])
	end
end

------------------------------------------------------------
-- Get the current id
------------------------------------------------------------
function choiceControlGetCurrentId(group)
	return group:GetUserData("currentId")
end

------------------------------------------------------------
-- Events
------------------------------------------------------------
-- Click on previous or next
function choiceControlApi_onClickOnPreviousNext(keyName, pressed, zoneEvent, event)
	local controlGroup = zoneEvent:GetZone():GetParent()
	if windowIsWinParentActive(controlGroup) then
		local delta = 1
		if zoneEvent:GetZone():GetName() == "btPreviousZone" then
			delta = -delta
		end
	
		if sndChoiceChange ~= nil then
			controlGroup:GetParentScreen():PlaySound(sndChoiceChange)
		end
		choiceControlChangeId(controlGroup, controlGroup:GetUserData("currentId") + delta)
	end
end
